#ifndef __GSGPU_HELPER_H__
#define __GSGPU_HELPER_H__

#include "gsgpu_drm.h"
#include <drm/gpu_scheduler.h>
#include <drm/drm_syncobj.h>
#include <drm/drm_cache.h>
#include <drm/drm_file.h>

#include "gsgpu.h"

/*
 * drm_sched_hw_job_reset() was replaced with drm_seched_stop.
 *
 * fix case 7.
 */
static inline void lg_drm_sched_stop(struct drm_gpu_scheduler *sched,
				     struct drm_sched_job *bad)
{
#if defined(LG_DRM_SCHED_STOP_PRESENT)
	drm_sched_stop(sched, bad);
#else
	drm_sched_hw_job_reset(sched, bad);
#endif
}

/*
 * Use drm_sched_job_cleanup() .
 *
 */
static inline void lg_drm_sched_job_cleanup(struct drm_sched_job *job)
{
#if defined(LG_DRM_SCHED_JOB_CLEANUP_PRESENT)
	drm_sched_job_cleanup(job);
#endif
}

/*
 * If the function drm_fb_helper_fill_info() is not present,
 * implement it.
 *
 * fix case 22.
 */
static inline void lg_drm_fb_helper_fill_info(struct fb_info *info,
					      struct drm_fb_helper *fb_helper,
					      struct drm_fb_helper_surface_size *sizes)
{
#if defined(LG_DRM_FB_HELPER_FILL_INFO_PRESENT)
	drm_fb_helper_fill_info(info, fb_helper, sizes);
#else
	struct drm_framebuffer *fb = fb_helper->fb;

	drm_fb_helper_fill_fix(info, fb->pitches[0], fb->format->depth);
	drm_fb_helper_fill_var(info, fb_helper,
			       sizes->fb_width, sizes->fb_height);

	info->par = fb_helper;
	snprintf(info->fix.id, sizeof(info->fix.id), "%sdrmfb",
		 fb_helper->dev->driver->name);
#endif
}

/*
 * fix case 1/10/13
 */
static inline int lg_gsgpu_ttm_bo_device_init(struct gsgpu_device *adev,
					      struct ttm_bo_driver *driver)
{
#if defined(LG_TTM_BO_DEVICE_INIT_HAS_GLOB_ARG)
	return ttm_bo_device_init(&adev->mman.bdev,
				  adev->mman.bo_global_ref.ref.object,
				  driver,
				  adev->ddev->anon_inode->i_mapping,
				  DRM_FILE_PAGE_OFFSET,
				  adev->need_dma32);
#else
	return ttm_bo_device_init(&adev->mman.bdev,
				  driver,
				  adev->ddev->anon_inode->i_mapping,
				  adev->need_dma32);
#endif
}

/*
 * fix case 25.
 */
static inline int lg_ttm_eu_reserve_buffers(struct ww_acquire_ctx *ticket,
					    struct list_head *list, bool intr,
					    struct list_head *dups, bool del_lru)
{
#if defined(LG_TTM_EU_RESERVE_BUFFERS_HAS_DEL_LRU_ARG)
	return ttm_eu_reserve_buffers(ticket, list, intr, dups, del_lru);
#else
	return ttm_eu_reserve_buffers(ticket, list, intr, dups);
#endif
}

/*
 * fix case 26
 */
static inline int lg_drm_syncobj_find_fence(struct drm_file *file_private,
					    u32 handle, u64 point, u64 flags,
					    struct dma_fence **fence)
{
#if defined(LG_DRM_SYNCOBJ_FIND_FENCE_HAS_FLAGS_ARG)
	return drm_syncobj_find_fence(file_private, handle, point, flags, fence);
#else
	return drm_syncobj_find_fence(file_private, handle, fence);
#endif
}

/*
 * fix case 30
 */
#if !defined(LG_TTM_BO_MOVE_TO_LRU_TAIL_HAS_BULK_ARG)
/**
 * struct ttm_lru_bulk_move_pos
 *
 * @first: first BO in the bulk move range
 * @last: last BO in the bulk move range
 *
 * Positions for a lru bulk move.
 */
struct ttm_lru_bulk_move_pos {
        struct ttm_buffer_object *first;
        struct ttm_buffer_object *last;
};

/**
 * struct ttm_lru_bulk_move
 *
 * @tt: first/last lru entry for BOs in the TT domain
 * @vram: first/last lru entry for BOs in the VRAM domain
 * @swap: first/last lru entry for BOs on the swap list
 *
 * Helper structure for bulk moves on the LRU list.
 */
struct ttm_lru_bulk_move {
        struct ttm_lru_bulk_move_pos tt[TTM_MAX_BO_PRIORITY];
        struct ttm_lru_bulk_move_pos vram[TTM_MAX_BO_PRIORITY];
        struct ttm_lru_bulk_move_pos swap[TTM_MAX_BO_PRIORITY];
};
#endif

static inline void lg_ttm_bo_move_to_lru_tail(struct ttm_buffer_object *bo,
					      struct ttm_lru_bulk_move *bulk)
{
#if defined(LG_TTM_BO_MOVE_TO_LRU_TAIL_HAS_BULK_ARG)
	ttm_bo_move_to_lru_tail(bo, bulk);
#else
	ttm_bo_move_to_lru_tail(bo);
#endif
}

/*
 * fix case 35
 */
static inline int lg_drm_hdmi_avi_infoframe_from_display_mode(
						struct hdmi_avi_infoframe *frame,
						struct drm_connector *connector,
						const struct drm_display_mode *mode)
{
#if defined(LG_DRM_HDMI_AVI_INFOFRAME_FROM_DISPLAY_MODE_HAS_CONNECTOR_ARG)
	return drm_hdmi_avi_infoframe_from_display_mode(frame, connector, mode);
#else
	bool is_hdmi2_sink;

	/*
         * FIXME: sil-sii8620 doesn't have a connector around when
         * we need one, so we have to be prepared for a NULL connector.
         */
	if (!connector)
		is_hdmi2_sink = true;
	else
		is_hdmi2_sink = connector->display_info.hdmi.scdc.supported ||
					(connector->display_info.color_formats &
					 DRM_COLOR_FORMAT_YCRCB420);

	return drm_hdmi_avi_infoframe_from_display_mode(frame, mode, is_hdmi2_sink);
#endif
}

/*
 * fix case 5
 */
lg_dma_resv_t *lg_gsgpu_gem_prime_res_obj(struct drm_gem_object *obj);

#if defined(LG_DRM_DRIVER_HAS_GEM_PRIME_RES_OBJ)
#define LG_DRM_DRIVER_GEM_PRIME_RES_OBJ_CALLBACK \
		.gem_prime_res_obj = lg_gsgpu_gem_prime_res_obj,
#else
#define LG_DRM_DRIVER_GEM_PRIME_RES_OBJ_CALLBACK
#endif

/*
 * fix case 24
 */
static inline void lg_ttm_validate_buffer_set_shared(struct ttm_validate_buffer *valid_buf, unsigned int num_shared)
{
#if defined(LG_TTM_VALIDATE_BUFFER_HAS_NUM_SHARED)
	valid_buf->num_shared = num_shared;
#else
	valid_buf->shared = num_shared;
#endif
}

/*
 * fix case 34
 */
static inline bool lg_drm_crtc_state_async_flip(struct drm_crtc_state *state)
{
#if defined(LG_DRM_CRTC_STATE_HAS_ASYNC_FLIP)
	return state->async_flip;
#else
	return (state->pageflip_flags & DRM_MODE_PAGE_FLIP_ASYNC);
#endif
}

/*
 * fix case 32
 */
static inline bool lg_drm_need_swiotlb(int dma_bits)
{
#if defined(LG_DRM_NEED_SWIOTLB_PRESENT)
	return drm_need_swiotlb(dma_bits);
#else
	return drm_get_max_iomem() > ((u64)1 << dma_bits);
#endif
}

/*
 * fix case 4/28
 */
static inline struct dma_buf *lg_drm_gem_prime_export(struct drm_gem_object *gobj,
						      int flags)
{
#if defined(LG_DRM_DRIVER_GEM_PRIME_EXPORT_HAS_DEV_ARG)
	return drm_gem_prime_export(gobj->dev, gobj, flags);
#else
	return drm_gem_prime_export(gobj, flags);
#endif
}

struct dma_buf *lg_gsgpu_gem_prime_export(
#if defined(LG_DRM_DRIVER_GEM_PRIME_EXPORT_HAS_DEV_ARG)
					  struct drm_device *dev,
#endif
					  struct drm_gem_object *gobj,
					  int flags);

#endif /* __GSGPU_HELPER_H__ */
